<?php
/**
 * Smart Wishlist Api
 *
 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @version 1.3.3
 * @since 1.3.3
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'WLFMC_Api' ) ) {
	/**
	 * This class handle rest api for wishlist plugin
	 */
	class WLFMC_Api {

		/**
		 * Endpoint namespace.
		 *
		 * @var string
		 */
		protected const NAMESPACE = 'wlfmc';

		/**
		 * Route base.
		 *
		 * @var string
		 */
		protected const REST_BASE = 'v1';

		/**
		 * Load the new REST API wishlist endpoints
		 */
		public function register_routes() {

			register_rest_route(
				self::NAMESPACE . '/' . self::REST_BASE,
				'/call',
				array(
					array(
						'methods'             => WP_REST_Server::CREATABLE,
						'callback'            => array( 'WLFMC_Api', 'do_action' ),
						'permission_callback' => '__return_true',
					),
				)
			);
		}

		/**
		 * Manage all rest_api ajax mode actions
		 *
		 * @param WP_REST_Request $request  The request object.
		 *
		 * @return WP_Error|WP_HTTP_Response|WP_REST_Response
		 */
		public static function do_action( WP_REST_Request $request ) {

			$data   = $request->get_body_params();
			$action = sanitize_key( $data['action'] );

			if ( ! in_array(
				$action,
				array(
					'wlfmc_wp_rest_add_to_wishlist',
					'wlfmc_wp_rest_remove_from_wishlist',
					'wlfmc_wp_rest_update_item_quantity',
					'wlfmc_wp_rest_delete_item',
					'wlfmc_wp_rest_load_fragments',
					'wlfmc_wp_rest_load_analytics',
				),
				true
			) ) {
				return rest_ensure_response( array( 'result' => false ) );
			}

			$action = str_replace( 'wlfmc_wp_rest_', '', $action );

			return self::$action( $data );

		}

		/**
		 * Add new hook for work analytics after product added to wishlist
		 *
		 * @param array $data $_POST data.
		 *
		 * @return void
		 */
		public static function load_analytics( array $data ) {
			$product_id  = isset( $data['product_id'] ) ? intval( $data['product_id'] ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$wishlist_id = isset( $data['wishlist_id'] ) ? intval( $data['wishlist_id'] ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			if ( ! $wishlist_id || ! $product_id ) {
				die();
			}

			do_action( 'wlfmc_analytics_added_to_wishlist', $product_id, $wishlist_id, get_current_user_id() );

			// stops ajax call from further execution (no return value expected on answer body).
			die();
		}

		/**
		 * Add to wishlist from ajax call
		 *
		 * @param array $data $_POST data.
		 *
		 * @return WP_Error|WP_HTTP_Response|WP_REST_Response
		 */
		public static function add_to_wishlist( array $data ) {

			$result = false;

			$args = array(
				'add_to_wishlist'     => 0,
				'wishlist_id'         => 0,
				'quantity'            => 1,
				'user_id'             => false,
				'dateadded'           => '',
				'wishlist_name'       => '',
				'wishlist_visibility' => 0,
			);
			$args = wp_parse_args( $data, $args );

			try {
				$result  = WLFMC()->add( $args );
				$return  = 'true';
				$message = '';

			} catch ( WLFMC_Exception $e ) {
				$return  = $e->getTextualCode();
				$message = apply_filters( 'wlfmc_error_adding_to_wishlist_message', $e->getMessage() );
			} catch ( Exception $e ) {
				$return  = 'error';
				$message = apply_filters( 'wlfmc_error_adding_to_wishlist_message', $e->getMessage() );
			}
			$product_id    = isset( $data['add_to_wishlist'] ) ? intval( $data['add_to_wishlist'] ) : false; // phpcs:ignore WordPress.Security
			$fragments     = isset( $data['fragments'] ) ? wp_unslash( $data['fragments'] ) : false;
			$found_item    = false;
			$found_in_list = false;
			if ( 'exists' === $return ) {
				$found_in_list = wlfmc_get_wishlist( false );
				$found_item    = $found_in_list ? $found_in_list->get_product( $product_id ) : false;
			}
			$wishlist_url = is_user_logged_in() ? apply_filters( 'wlfmc_user_logged_in_wishlist_url', wc_get_account_endpoint_url( get_option( 'woocommerce_myaccount_wlfmc_wishlist_endpoint', 'wlfmc-wishlist' ) ) ) : WLFMC()->get_last_operation_url();

			return rest_ensure_response(
				apply_filters(
					'wlfmc_rest_add_return_params',
					array(
						'prod_id'      => $product_id,
						'result'       => $return,
						'message'      => $message,
						'wishlist_url' => $wishlist_url,
						'fragments'    => WLFMC_Ajax_Handler::refresh_fragments( $fragments ),
						'wishlist_id'  => $result ? $result['wishlist_id'] : ( $found_in_list ? $found_in_list->get_id() : false ),
						'item_id'      => $result ? $result['item_id'] : ( $found_item ? $found_item->get_id() : false ),
					)
				)
			);
		}

		/**
		 * Remove item from a wishlist
		 *
		 * @param array $data $_POST data.
		 *
		 * @return WP_Error|WP_HTTP_Response|WP_REST_Response
		 */
		public static function remove_from_wishlist( array $data ) {
			$fragments = isset( $data['fragments'] ) ? wp_unslash( $data['fragments'] ) : false;
			$return    = false;
			$result    = false;

			$args = array(
				'remove_from_wishlist' => 0,
				'wishlist_id'          => 0,
				'user_id'              => false,
			);
			$args = wp_parse_args( $data, $args );
			try {
				$result  = WLFMC()->remove( $args );
				$return  = 'true';
				$message = '';
			} catch ( Exception $e ) {
				$message = $e->getMessage();
			}

			return rest_ensure_response(
				array(
					'result'    => $return,
					'message'   => $message,
					'count'     => $result ? $result['count'] : false,
					'fragments' => WLFMC_Ajax_Handler::refresh_fragments( $fragments ),
				)
			);
		}

		/**
		 * Remove item from a wishlist
		 * Differs from remove from wishlist, since this accepts item id instead of product id
		 *
		 * @param array $data $_POST data.
		 *
		 * @return WP_Error|WP_HTTP_Response|WP_REST_Response
		 */
		public static function delete_item( array $data ) {

			$item_id   = isset( $data['item_id'] ) ? intval( $data['item_id'] ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$fragments = isset( $data['fragments'] ) ? wp_unslash( $data['fragments'] ) : false;
			$return    = array(
				'result' => false,
			);

			if ( $item_id ) {
				$item = WLFMC_Wishlist_Factory::get_wishlist_item( $item_id );

				if ( $item ) {
					$item->delete();
					$message = '';

					$return = array(
						'result'    => 'true',
						'message'   => $message,
						'fragments' => WLFMC_Ajax_Handler::refresh_fragments( $fragments ),
					);
				}
			}

			return rest_ensure_response( $return );
		}

		/**
		 * Update quantity of an item in wishlist
		 *
		 * @param array $data $_POST data.
		 *
		 * @return void
		 */
		public static function update_item_quantity( array $data ) {
			$wishlist_token = isset( $data['wishlist_token'] ) ? sanitize_text_field( wp_unslash( $data['wishlist_token'] ) ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$product_id     = isset( $data['product_id'] ) ? intval( $data['product_id'] ) : false; // phpcs:ignore WordPress.Security.NonceVerification
			$quantity       = isset( $data['quantity'] ) ? intval( $data['quantity'] ) : 1; // phpcs:ignore WordPress.Security.NonceVerification

			if ( ! $wishlist_token || ! $product_id ) {
				die();
			}

			$wishlist = wlfmc_get_wishlist( $wishlist_token );

			if ( ! $wishlist || ! $wishlist->current_user_can( 'update_quantity' ) ) {
				die();
			}

			$item = $wishlist->get_product( $product_id );

			if ( ! $item ) {
				die();
			}

			$item->set_quantity( $quantity );
			$item->save();

			// stops ajax call from further execution (no return value expected on answer body).
			die();
		}

		/**
		 * Generated fragments to replace in the page
		 *
		 * @param array $data $_POST data.
		 *
		 * @return WP_Error|WP_HTTP_Response|WP_REST_Response
		 */
		public static function load_fragments( array $data ) {
			$fragments = isset( $data['fragments'] ) ? wp_unslash( $data['fragments'] ) : false;
			if ( defined( 'ICL_SITEPRESS_VERSION' ) ) { // wpml current  language.
				global $sitepress;
				$lang = $sitepress->get_current_language();
			} elseif ( function_exists( 'pll_current_language' ) ) { // polylang current language.
				$lang = pll_current_language();
			} else { // cannot determine current language.
				$lang = null;
			}

			return rest_ensure_response(
				array(
					'fragments' => WLFMC_Ajax_Handler::refresh_fragments( $fragments ),
					'products'  => WLFMC_Ajax_Handler::get_current_items(),
					'lang'      => $lang,
				)
			);
		}
	}

}

add_action( 'rest_api_init', 'wlfmc_api_init' );

/**
 * Run API routes
 *
 * @return void
 */
function wlfmc_api_init() {

	$controller = new WLFMC_Api();
	$controller->register_routes();

}




