<?php
/**
 * Static class that will handle all ajax calls for the list
 *
 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @version 1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}// Exit if accessed directly

if ( ! class_exists( 'WLFMC_Automation_Ajax_Handler' ) ) {
	/**
	 * Woocommerce Smart Wishlist Ajax Handler
	 */
	class WLFMC_Automation_Ajax_Handler {
		/**
		 * Performs all required add_actions to handle forms
		 *
		 * @return void
		 */
		public static function init() {
			add_action(
				'wp_ajax_wlfmc_reset_sending_cycles_automation',
				array(
					'WLFMC_Automation_Ajax_Handler',
					'reset_sending_cycles',
				)
			);
			add_action( 'wp_ajax_wlfmc_save_automation', array( 'WLFMC_Automation_Ajax_Handler', 'save_automation' ) );
			add_action( 'wp_ajax_wlfmc_new_automation', array( 'WLFMC_Automation_Ajax_Handler', 'new_automation' ) );
			add_action(
				'wp_ajax_wlfmc_send_offer_email_test',
				array(
					'WLFMC_Automation_Ajax_Handler',
					'test_callback',
				)
			);
		}

		/**
		 * Reset sending cycles
		 *
		 * @return void
		 */
		public function reset_sending_cycles() {
			global $wpdb;

			check_ajax_referer( 'ajax-nonce', 'key' );

			if ( ! apply_filters( 'wlfmc_can_reset_sending_cycle', true ) ) {
				wp_send_json(
					array(
						'message' => __( 'You cannot access to reset sending cycle', 'wc-wlfmc-wishlist' ),
						'success' => false,
					)
				);
			}
			$deleted = $wpdb->delete( $wpdb->usermeta, array( 'meta_key' => 'wlfmc_last_period_days' ) );// phpcs:ignore WordPress.DB

			if ( ! is_wp_error( $deleted ) ) {
				echo wp_json_encode(
					array(
						'message' => __( 'All sending cycles reset.', 'wc-wlfmc-wishlist' ),
						'success' => true,
					)
				);
			} else {
				echo wp_json_encode(
					array(
						'message' => __( 'Problem on reset sending cycle, try again later', 'wc-wlfmc-wishlist' ),
						'success' => false,
					)
				);
			}

			exit;
		}

		/**
		 * Send offer email for testing
		 */
		public static function test_callback() {
			check_ajax_referer( 'ajax-nonce', 'key' );

			if ( ! apply_filters( 'wlfmc_can_send_test_mail', true ) ) {
				wp_send_json(
					array(
						'message' => __( 'You cannot access to send test mail', 'wc-wlfmc-wishlist' ),
						'success' => false,
					)
				);
			}

			$id            = isset( $_POST['id'] ) ? absint( $_POST['id'] ) : '';
			$automation_id = isset( $_POST['automation_id'] ) ? absint( $_POST['automation_id'] ) : '';

			if ( '' === $automation_id || '' === $id ) {
				exit;
			}

			$automation = new WLFMC_Automation( $automation_id );

			$options = $automation->get_options();

			$email_type = $options['mail-type'] ?? '';
			$offers     = $options['offer_emails'] ?? '';

			if ( ! $offers || ! isset( $offers[ $id ] ) ) {

				echo wp_json_encode(
					array(
						'message' => __( 'Save the email settings first, then try again', 'wc-wlfmc-wishlist' ),
						'success' => false,
					)
				);
				exit;
			}

			$mailer        = WC()->mailer();
			$wlfmc_email   = new WLFMC_Email();
			$user          = get_userdata( get_current_user_id() );
			$to            = $user->user_email;
			$email_options = $offers[ $id ];
			$email_content = '';
			$content_type  = '';
			$mc_options    = array();
			$coupon_amount = $options['coupon-amount'];
			$template      = '';

			switch ( $email_type ) {
				case 'mc-template':
					$content_type = 'text/html';
					$template     = 'mc-template.php';

					$email_content = ( '' === $email_options['html_content'] ) ? $automation::get_default_content( 'html', $id ) : $email_options['html_content'];
					$logo          = wp_get_attachment_image_src( $options['email-template-logo'] );
					$avatar        = wp_get_attachment_image_src( $options['email-template-avatar'] );

					$mc_options = array(
						'logo'          => $logo ? $logo : '',
						'avatar'        => $avatar ? $avatar : '',
						'customer-name' => $options['email-template-customer-name'],
						'customer-job'  => $options['email-template-customer-job'],
						'facebook'      => $options['email-template-facebook'],
						'linkedin'      => $options['email-template-linkedin'],
						'instagram'     => $options['email-template-instagram'],
					);
					break;
				case 'html':
					$content_type  = 'text/html';
					$template      = 'offer.php';
					$email_content = ( '' === $email_options['text_content'] ) ? $automation::get_default_content( 'html', $id ) : $email_options['html_content'];
					break;
				case 'plain':
					$content_type  = 'text/plain';
					$template      = 'plain/offer.php';
					$email_content = ( '' === $email_options['text_content'] ) ? $automation::get_default_content( 'plain', $id ) : $email_options['text_content'];

					break;
			}

			$email_content = $wlfmc_email->prepare_content( $email_content, $email_type );

			$headers = "Content-Type: {$content_type}\r\n";

			$placeholders = array(
				'{user_name}'        => $user->user_login,
				'{user_email}'       => $user->user_email,
				'{user_first_name}'  => $user->first_name,
				'{user_last_name}'   => $user->last_name,
				'{site_name}'        => get_bloginfo( 'name' ),
				'{coupon_amount}'    => $coupon_amount,
				'{site_description}' => get_bloginfo( 'description' ),

			);
			$email_content = str_replace( array_keys( $placeholders ), array_values( $placeholders ), $email_content );

			$email_heading = str_replace( array_keys( $placeholders ), array_values( $placeholders ), $email_options['mail_heading'] );
			$email_subject = str_replace( array_keys( $placeholders ), array_values( $placeholders ), $email_options['mail_subject'] );

			$message = wlfmc_get_template(
				'emails/' . $template,
				array(
					'email'         => $mailer,
					'email_heading' => apply_filters( 'wlfmc_automation_email_heading', $email_heading ),
					'email_content' => apply_filters( 'wlfmc_automation_email_content', $email_content ),
					'mc_options'    => $mc_options,
					'wishlist_url'  => '#!',
				),
				true
			);

			add_filter( 'woocommerce_email_from_name', array( $automation, 'get_from_name' ), 10 );
			add_filter( 'woocommerce_email_from_address', array( $automation, 'get_from_address' ), 10 );
			add_filter( 'woocommerce_email_content_type', array( $automation, 'get_content_type' ), 10 );

			$send_state = $mailer->send( $to, apply_filters( 'wlfmc_automation_email_subject_test', $email_subject ), $message, $headers, '' );

			remove_filter( 'woocommerce_email_from_name', array( $automation, 'get_from_name' ), 10 );
			remove_filter( 'woocommerce_email_from_address', array( $automation, 'get_from_address' ), 10 );
			remove_filter( 'woocommerce_email_content_type', array( $automation, 'get_content_type' ), 10 );

			if ( $send_state ) {
				echo wp_json_encode(
					array(
						/* translators: %s: email address */
						'message' => sprintf( __( 'Email sent to %s.', 'wc-wlfmc-wishlist' ), $to ),
						'success' => true,
					)
				);
			} else {
				echo wp_json_encode(
					array(
						'message' => __( 'There was a problem sending the email.', 'wc-wlfmc-wishlist' ),
						'success' => false,
					)
				);
			}

			exit;
		}

		/**
		 * Create new automation
		 *
		 * @return void
		 */
		public static function new_automation() {

			global $wpdb;

			check_ajax_referer( 'ajax-nonce', 'key' );

			if ( ! apply_filters( 'wlfmc_can_create_automation', true ) ) {
				wp_send_json(
					array(
						'errors' => __( 'You cannot access to create new automation', 'wc-wlfmc-wishlist' ),
						'status' => false,
					)
				);
			}
			$data = array(
				'status'         => false,
				'errors'         => array(),
				'show_pro_popup' => false,
			);

			$automation_count = (int) $wpdb->get_var( "SELECT  COUNT(ID) FROM {$wpdb->wlfmc_automations}" );// phpcs:ignore WordPress.DB

			if ( $automation_count > 0 ) {
				$data['show_pro_popup'] = true;
			}

			$options = apply_filters(
				'wlfmc_ajax_automation_options',
				array(
					'automation_name' => '',
					'list_type'       => 'default',
					'is_active'       => 0,
					'options'         => array(),
				)
			);

			if ( ! empty( $options ) && 0 === $automation_count ) {
				$automation = new WLFMC_Automation();
				$automation->set_name( $options['automation_name'] );
				$automation->set_list_type( $options['list_type'] );
				$automation->set_status( $options['is_active'] );
				$automation->set_options( $options );
				// insert.
				$insert = $automation->insert();

				if ( $insert ) {
					$data['status'] = true;
					$data['url']    = add_query_arg(
						array(
							'page'          => 'mc-wishlist-marketing',
							'tools'         => 'email-automation',
							'tools-action'  => 'edit',
							'automation_id' => $automation->get_id(),
						),
						admin_url( 'admin.php' )
					);
				}
			}
			wp_send_json( apply_filters( 'wlfmc_ajax_new_automation', $data, $options ) );

		}

		/**
		 * Save automation as draft
		 *
		 * @return void
		 */
		public static function save_automation() {

			check_ajax_referer( 'ajax-nonce', 'key' );

			if ( ! isset( $_POST['options'] ) ) {
				return;
			}
			if ( ! apply_filters( 'wlfmc_can_update_automation', true ) ) {
				wp_send_json(
					array(
						'errors' => __( 'You cannot access to update automation', 'wc-wlfmc-wishlist' ),
						'status' => false,
					)
				);
			}

			parse_str( wp_unslash( $_POST['options'] ), $options ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized

			$data    = array(
				'status' => false,
				'errors' => array(),
			);
			$options = wp_parse_args(
				$options,
				apply_filters(
					'wlfmc_ajax_automation_options',
					array(
						'automation_name' => '',
						'list_type'       => 'default',
						'is_active'       => 0,
					)
				)
			);
			// Handle required fields.
			$required_fields = apply_filters(
				'wlfmc_save_automation_required_fields',
				array(
					'automation_name'    => __( 'Automation name', 'wc-wlfmc-wishlist' ),
					'email-from-name'    => __( 'From "name"', 'wc-wlfmc-wishlist' ),
					'email-from-address' => __( 'From "Email address"', 'wc-wlfmc-wishlist' ),
				)
			);

			foreach ( $required_fields as $field_key => $field_name ) {
				if ( empty( $options[ $field_key ] ) ) {
					/* translators: %s: field name'*/
					$data['errors'][] = sprintf( __( '%s is a required field.', 'wc-wlfmc-wishlist' ), '<strong>' . esc_html( $field_name ) . '</strong>' );
				}
			}
			if ( ! empty( $options ) && empty( $data['errors'] ) ) {

				$automation_id = isset( $options['automation_id'] ) ? absint( $options['automation_id'] ) : 0;
				unset( $options['automation_id'] );
				$automation = new WLFMC_Automation( $automation_id );

				if ( 0 < $automation->get_id() ) {
					// update.
					$automation->set_name( $options['automation_name'] );
					$automation->set_list_type( $options['list_type'] );
					$automation->set_status( $options['is_active'] );
					$automation->set_options( $options );
					$automation->save();
					$data['status'] = true;
					$data['url']    = add_query_arg(
						array(
							'page'  => 'mc-wishlist-marketing',
							'tools' => 'email-automation',
						),
						admin_url( 'admin.php' )
					);
				} else {
					$data['errors'][] = __( 'This automation not exists.', 'wc-wlfmc-wishlist' );

				}
			}
			wp_send_json( apply_filters( 'wlfmc_ajax_save_automation', $data, $options ) );

		}


	}
}

if ( defined( 'MC_WLFMC_PREMIUM' ) ) {

	WLFMC_Automation_Ajax_Handler_Premium::init();

} else {

	WLFMC_Automation_Ajax_Handler::init();

}

